import re
import sys

from sudachipy import dictionary

# 漢字を表す正規表現
KANJI = re.compile(r"[\u3005-\u3007\u4E00-\u9FFF]")

# すべてのひらがなとカタカナを含む正規表現
KANA = re.compile(r"[\u3041-\u309F\u30A1-\u30FF]+")

# カタカナをひらがなに変換
K2H = str.maketrans(
    "アイウエオカキクケコサシスセソタチツテトナニヌネノハヒフヘホマミムメモヤユヨラリルレロワヰヱヲンガギグゲゴザジズゼゾダヂヅデドバビブベボパピプペポァィゥェォヵヶッャュョヮ",
    "あいうえおかきくけこさしすせそたちつてとなにぬねのはひふへほまみむめもやゆよらりるれろわゐゑをんがぎぐげござじずぜぞだぢづでどばびぶべぼぱぴぷぺぽぁぃぅぇぉゕゖっゃゅょゎ",
)

t = dictionary.Dictionary(config_path="sudachi.json").create()


def make_ruby(k: str, f: str) -> str:
    """rubyタグを生成して返す"""
    return f"<ruby><rb>{k}</rb><rt>{f}</rt></ruby>"


def ruby(kanji: str, kana: str) -> str:
    """1つの単語にフリガナを振る

    >>> ruby("麦酒", "びーる")
    '<ruby><rb>麦酒</rb><rt>びーる</rt></ruby>'
    >>> ruby("飲もう", "のもう")
    '<ruby><rb>飲</rb><rt>の</rt></ruby>もう'
    >>> ruby("追い出す", "おいだす")
    '<ruby><rb>追</rb><rt>お</rt></ruby>い<ruby><rb>出</rb><rt>だ</rt></ruby>す'
    >>> ruby("しみ込む", "しみこむ")
    'しみ<ruby><rb>込</rb><rt>こ</rt></ruby>む'
    >>> ruby("東アジア", "ひがしあじあ")
    '<ruby><rb>東</rb><rt>ひがし</rt></ruby>アジア'
    """
    kana = kana.translate(K2H)
    text = ""
    # 漢字の中のかなをすべて抜き出す
    while m := KANA.search(kanji):
        okuri = m[0]
        # フリガナで最初にかなが出る位置を返す
        index = kana.find(okuri.translate(K2H), m.start())
        ruby = kana[:index]
        # 残りのフリガナ
        kana = kana[index + len(okuri):]
        # 漢字を送り仮名で分割
        f_kanji, kanji = kanji.split(okuri, 1)
        if ruby:
            text += make_ruby(f_kanji, ruby)
        text += okuri
    if kanji:  # 漢字が残っている場合
        text += make_ruby(kanji, kana)

    return text


def furigana(s: str) -> str:
    """指定された文字列にフリガナを振る"""
    result = ""
    for token in t.tokenize(s):
        surface = token.surface()
        # 漢字のときはフリガナを振る
        if KANJI.search(surface):
            result += ruby(surface, token.reading_form())
        else:
            result += surface
    return result


if __name__ == "__main__":
    print(furigana(sys.argv[1]))
